
/////////////////////////////////////////////////////////////////////
// DESP-C++ (Discrete-Event Simulation Package)
// Version 1.1, February 1998
// Jerome Darmont
// LIMOS, Blaise Pascal University (Clermont-Ferrand II), France
/////////////////////////////////////////////////////////////////////
// eventc.h: Event Manager methods definition
// Variable with simulated systems
// => Flowshop sample (sample_eventc.h)
/////////////////////////////////////////////////////////////////////

#include "stdlib.h"
#include "string.h"

/////////////////////////////////////////////////////////////////////
// CLASS EventManager
/////////////////////////////////////////////////////////////////////

// CLASS EventManager: Constructor

EventManager::EventManager(Simulation *sim) {

  simul=sim;

  // Passive resources instantiation

  robot=new Resource("Robot",1,simul);

  // Active resources instantiation

  machine2=new Machine2("Machine #2",1,simul,robot);
  machine1=new Machine1("Machine #1",1,simul,robot,machine2);
}

// CLASS EventManager: Destructor

EventManager::~EventManager() {

  // Passive resources destruction

  delete robot;

  // Active resources destruction

  delete machine1;
  delete machine2;
}

// CLASS EventManager: Events execution

void EventManager::ExecuteEvent(int code, Client *client) {

  switch(code) {

  case 0:  machine1->Event10(client);break;

  case 10: machine1->Event10(client);break;
  case 11: machine1->Event11(client);break;
  case 12: machine1->Event12(client);break;
  case 13: machine1->Event13(client);break;

  case 20: machine2->Event20(client);break;
  case 21: machine2->Event21(client);break;
  case 22: machine2->Event22(client);break;
  case 23: machine2->Event23(client);break;
  case 24: machine2->Event24(client);break;

  default: printf("Error: unknown event #%d at time %f\n",code,simul->Tnow());
  }
}

// CLASS EventManager: Stats init for each resource

void EventManager::Init() {

  // Passive resources
  robot->ResetStats();
  // Active resources
  machine1->ResetStats();
  machine2->ResetStats();
}

// CLASS EventManager: Replications initialization

void EventManager::InitRep() {

  // Scheduler
  simul->Sched()->Purge();
  // Passive resources
  robot->ResetCounters();
  robot->PurgeQueue();
  // Active resources
  machine1->ResetCounters();
  machine1->PurgeQueue();
  machine2->ResetCounters();
  machine2->PurgeQueue();
}

// CLASS EventManager: Stats computation for each resource

void EventManager::Stats() {

  // Passive resources
  robot->Stats();
  // Active resources
  machine1->Stats();
  machine2->Stats();
}

// CLASS EventManager: Stats display for each resource

void EventManager::DisplayStats() {

  printf("\n*** PASSIVE RESOURCES\n");
  robot->DisplayStats();
  printf("\n*** ACTIVE RESOURCES\n");
  machine1->DisplayStats();
  machine2->DisplayStats();
}

/////////////////////////////////////////////////////////////////////
// CLASS Machine1
/////////////////////////////////////////////////////////////////////

// CLASS Machine1: Constructor

Machine1::Machine1(char n[STRS], int cap, Simulation *sim, Resource *rob, Machine2 *suiv): Resource(n, cap, sim) {

  robot=rob;
  m2=suiv;
}

// CLASS Machine1: Event #10 - Arrival on M1

void Machine1::Event10(Client *client) {

//  printf("Waiting for M1 at time %f\n",Sim()->Tnow());
  this->P(11,client,1);
}

// CLASS Machine1: Event #11 - Begin product generation

void Machine1::Event11(Client *client) {

//  printf("Begin production on M1 at time %f\n",Sim()->Tnow());
  Sim()->Sched()->Schedule(12,Sim()->Tnow()+Exp(10),client);
}

// CLASS Machine1: Event #12 - Product out

void Machine1::Event12(Client *client) {

//  printf("End production on M1 at time %f - Waiting for robot\n",Sim()->Tnow());
  robot->P(13,client,1);
}

// CLASS Machine1: Event #13 - Transfer toward M2

void Machine1::Event13(Client *client) {

  Client *newclient;

//  printf("Release M1 - Transfer M1-M2 at time %f\n",Sim()->Tnow());
  this->V();
  newclient=Sim()->NewClient();
  Sim()->Sched()->Schedule(10,Sim()->Tnow(),newclient);
  Sim()->Sched()->Schedule(20,Sim()->Tnow()+Uni(4,6),client);
}

/////////////////////////////////////////////////////////////////////
// CLASS Machine2
/////////////////////////////////////////////////////////////////////

// CLASS Machine2: Constructor

Machine2::Machine2(char n[STRS], int cap, Simulation *sim, Resource *rob): Resource(n, cap, sim) {

  robot=rob;
  production=0;
}

// CLASS Machine2: Event #20 - Arrival on M2

void Machine2::Event20(Client *client) {

//  printf("Release robot - Waiting for M2 at time %f\n",Sim()->Tnow());
  robot->V();
  this->P(21,client,1);
}

// CLASS Machine2: Event #21 - Begin product transformation

void Machine2::Event21(Client *client) {

//  printf("Begin transformation on M2 at time %f\n",Sim()->Tnow());
  Sim()->Sched()->Schedule(22,Sim()->Tnow()+Exp(12),client);
}

// CLASS Machine2: Event #22 - End product transformation

void Machine2::Event22(Client *client) {

//  printf("End transformation on M2 at time %f - Waiting for robot\n",Sim()->Tnow());
  robot->P(23,client,1);
}

// CLASS Machine2: Event #23 - Transfer out of M2

void Machine2::Event23(Client *client) {

//  printf("Release M2 - Transfer out of M2 at time %f\n",Sim()->Tnow());
  this->V();
  Sim()->Sched()->Schedule(24,Sim()->Tnow()+Uni(4,6),client);
}

// CLASS Machine2: Event #24 - Transfer out of system

void Machine2::Event24(Client *client) {

//  printf("Release robot at time %f\n",Sim()->Tnow());
  robot->V();
  production++;
  Sim()->KillClient(client);
//  printf("Production=%d\n",production);
}
